"""Functions which translate the human readable, nested definition data from
the input yaml files into the flattened structures required by the conference
scheduler computation engine."""

from datetime import datetime
from datetime import timedelta

from conference_scheduler.resources import Event
from conference_scheduler.resources import Slot


def types_and_slots(timetable):
    """
    Parameters
    ----------
    timetable : dict
        of the form
            {
                <venue name>: {
                    <datetime of a day>: {
                        <session name>: {
                            'starts_at': <seconds after midnight>,
                            'duration': <minutes>,
                            'event_type': <event type>,
                            'capacity': <capacity
                        }
                    }
                }
            }
    Returns
    -------
    list
        of dicts mapping an event type to a Slot instance
    """
    return [
        {
            'event_type': slot['event_type'],
            'slot': Slot(
                venue=venue,
                starts_at=(
                    datetime.combine(day, datetime.min.time()) +
                    timedelta(seconds=slot['starts_at'])),
                duration=slot.get('duration', 0),
                session=f'{day} {session}',
                capacity=slot.get('capacity', 0))
        }
        for venue, days in timetable.items()
        for day, sessions in days.items()
        for session, slots in sessions.items()
        for slot in slots
    ]


def types_and_events(events_definition):
    """
    Parameters
    ----------
    events_definition : list
        of dicts of the form
            {'title': Event title,
            'duration': <integer in minutes>,
            'tags': <list of strings>,
            'person': <string>}
    Returns
    -------
    list
        of dicts mapping an event type to an Event instance
    """
    return [
        {
            'event_type': event['event_type'],
            'event': Event(
                name=event['title'],
                duration=event['duration'],
                demand=event['demand'],
                tags=event['tags'])
        }
        for event in events_definition]


def people_unavailability(events_definition, slots, unavailability_definition):
    """
    Parameters
    ----------
    events_definition : list
        of dicts of the form
            {'title': Event title,
            'duration': <integer in minutes>,
            'tags': <list of strings>,
            'person': <string>,
            'event_type': <string>}
    slots : list
        of Slot instances
    unavailablity_definition : dict
        mapping a person to a list of time periods. e.g.
            {'owen-campbell': [{
                'unavailable_from': datetime(2017, 10, 26, 0, 0),
                'unavailable_until': datetime(2017, 10, 26, 23, 59)}]
            }

    Returns
    -------
    dict
        mapping the index of an event in the events list to a list of slots
        for which it must not scheduled. The slots are represented by their
        index in the slots list.
    """
    return {
        events_definition.index(event): [
            slots.index(slot)
            for period in periods
            for slot in slots
            if period['unavailable_from'] <= slot.starts_at and
            period['unavailable_until'] >= (
                slot.starts_at + timedelta(0, slot.duration * 60))
        ]
        for person, periods in unavailability_definition.items()
        for event in events_definition if event['person'] == person
    }


def events_unavailability(events_definition, slots, unavailability_definition):
    """
    Parameters
    ----------
    events_definition : list
        of dicts of the form
            {'title': Event title,
            'duration': <integer in minutes>,
            'tags': <list of strings>,
            'person': <string>,
            'event_type': <string>}
    slots : list
        of Slot instances
    unavailablity_definition : dict
        mapping an event to a list of time periods. e.g.
            {'An Interesting Talk': [{
                'unavailable_from': datetime(2017, 10, 26, 0, 0),
                'unavailable_until': datetime(2017, 10, 26, 23, 59)}]
            }

    Returns
    -------
    dict
        mapping the index of an event in the events list to a list of slots
        for which it must not scheduled. The slots are represented by their
        index in the slots list.
    """
    return {
        events_definition.index(event): [
            slots.index(slot)
            for period in periods
            for slot in slots
            if period['unavailable_from'] <= slot.starts_at and
            period['unavailable_until'] >= (
                slot.starts_at + timedelta(0, slot.duration * 60))
        ]
        for event_title, periods in unavailability_definition.items()
        for event in events_definition if event['title'] == event_title
    }


def people_clashes(events_definition, clashes_definition):
    """
     Parameters
    ----------
    events_definition : list
        of dicts of the form
            {'title': Event title,
            'duration': <integer in minutes>,
            'tags': <list of strings>,
            'person': <string>,
            'event_type': <string>}
    clashes_definition : dict
        mapping a person to a list of people whose events they must not not be
        scheduled against.

    Returns
    -------
    dict
        mapping the index of an event in the events list to a list of event
        indexes against which it must not be scheduled.
    integer
        the count of self-clashes added
    """
    # Add everyone who is missing to the clashes definition so that they cannot
    # clash with themselves
    for person in [event['person'] for event in events_definition]:
        if person not in clashes_definition:
            clashes_definition[person] = [person]

    # Add the self-clashing constraint to any existing entries where it is
    # missing
    count = 0
    for person, clashing_people in clashes_definition.items():
        if person not in clashing_people:
            clashing_people.append(person)
            count += 1

    clashes = {
        events_definition.index(event): [
            events_definition.index(t) for c in clashing_people
            for t in events_definition
            if t['person'] == c and
            events_definition.index(event) != events_definition.index(t)]
        for person, clashing_people in clashes_definition.items()
        for event in events_definition if event['person'] == person
    }
    return clashes, count


def events_clashes(events_definition, clashes_definition):
    """
     Parameters
    ----------
    events_definition : list
        of dicts of the form
            {'title': Event title,
            'duration': <integer in minutes>,
            'tags': <list of strings>,
            'person': <string>,
            'event_type': <string>}
    clashes_definition : dict
        mapping an event to a list of events they must not not be
        scheduled against.

    Returns
    -------
    dict
        mapping the index of an event in the events list to a list of event
        indexes against which it must not be scheduled.
    """
    clashes = {
        events_definition.index(event): [
            events_definition.index(t) for c in clashing_events
            for t in events_definition
            if t['title'] == c and
            events_definition.index(event) != events_definition.index(t)]
        for event_title, clashing_events in clashes_definition.items()
        for event in events_definition if event['title'] == event_title
    }
    return clashes


def unsuitability(types_and_slots, events_definition):
    """
    Parameters
    ----------
    types_and_slots : list
        of dicts mapping an event type to a Slot instance
    events_definition : list
        of dicts of the form
            {'title': Event title,
            'duration': <integer in minutes>,
            'tags': <list of strings>,
            'person': <string>,
            'event_type': <string>}
    Returns
    -------
    dict
        mapping the index of an event in the events list to a list of slots
        for which it must not scheduled. The slots are represented by their
        index in the slots list.
    """
    output = {}
    for i, event in enumerate(events_definition):
        unsuitable_slots = [
            i for i, dictionary in enumerate(types_and_slots)
            if dictionary['event_type'] != event['event_type']]
        output[i] = unsuitable_slots
    return output


def allocations(allocations_definition):
    try:
        return [
            {
                'event': Event(
                    name=event,
                    duration=0,
                    demand=0,
                    tags=details['tags']),
                'slot': Slot(
                    venue=details['venue'],
                    starts_at=(datetime.combine(
                        details['day'],
                        datetime.min.time()) +
                        timedelta(seconds=details['starts_at'])),
                    duration=0,
                    session=(f'{details["day"]} {details["session"]}'),
                    capacity=0)
            }
            for allocation in allocations_definition
            for event, details in allocation.items()]
    except TypeError:
        return []



"""Functions and procedures to construct the definition of the conference
in a form suitable for the scheduling enging."""
from collections import Counter
from pprint import pformat

import daiquiri

import scheduler.denormalise as dn
from scheduler import io, session

logger = daiquiri.getLogger(__name__)


def resources():
    resources = io.import_yaml()
    resources['events'] = io.import_proposals(resources)
    return resources


def slots(resources):

    types_and_slots = dn.types_and_slots(resources['timetable'])
    logger.debug(f'\ntypes_and_slots:\n{pformat(types_and_slots)}')

    event_types = Counter([item['event_type'] for item in types_and_slots])
    for event_type, count in event_types.items():
        logger.info(f'{count} {event_type} slots available')
    return [item['slot'] for item in types_and_slots]


def events(resources):
    events = dn.types_and_events(resources['events'])
    logger.debug(f'\nevents:\n{pformat(events)}')

    event_types = Counter([item['event_type'] for item in events])
    for event_type, count in event_types.items():
        logger.info(f'{count} {event_type} events to schedule')
    return [item['event'] for item in events]


def unavailability(resources, slots):
    try:
        people_unavailability = dn.people_unavailability(
            resources['events'], slots, resources['unavailability']['people'])
    except KeyError:
        people_unavailability = {}
    logger.debug(f'\npeople_unavailability:\n{people_unavailability}')
    logger.info(f'{len(people_unavailability)} person(s) with unavailability')

    try:
        events_unavailability = dn.events_unavailability(
            resources['events'], slots, resources['unavailability']['events'])
    except KeyError:
        events_unavailability = {}
    logger.debug(f'\nevents_unavailability:\n{events_unavailability}')
    logger.info(f'{len(events_unavailability)} event(s) with unavailability')

    return {**people_unavailability, **events_unavailability}


def clashes(resources):
    try:
        people_clashes, count = dn.people_clashes(
            resources['events'], resources['clashes']['people'])
    except KeyError:
        people_clashes, count = {}, 0
    logger.debug(f'\npeople_clashes:\n{clashes}')
    logger.info(f'{count} person(s) with clashes')

    try:
        events_clashes = dn.events_clashes(
            resources['events'], resources['clashes']['events'])
    except KeyError:
        events_clashes = {}
    logger.debug(f'\nevents_clashes:\n{clashes}')
    logger.info(f'{count} event(s) with clashes')

    return {**people_clashes, **events_clashes}


def unsuitability(resources, slots):

    types_and_slots = dn.types_and_slots(resources['timetable'])

    unsuitability = dn.unsuitability(types_and_slots, resources['events'])
    logger.debug(f'\nunsuitability:\n{unsuitability}')
    logger.info(f'{len(unsuitability)} events with unsuitable timetable')
    return unsuitability


def allocations(resources):
    allocations = dn.allocations(resources['allocations'])
    logger.debug(f'\nallocations\n{allocations}')
    logger.info(f'{len(allocations)} pre-allocated event(s)')
    return allocations


def add_unavailability_to_events(events, slots, unavailability):
    for event, unavailable_slots in unavailability.items():
        events[event].add_unavailability(
            *[slots[s] for s in unavailable_slots])
    # logger.debug(f'\nevents with unavailability added:\n{pformat(events)}')
    logger.info(
        f'Added unavailability for {len(unavailability)} events.')


def add_clashes_to_events(events, clashes):
    for event, clashing_events in clashes.items():
        events[event].add_unavailability(*[events[t] for t in clashing_events])
    # logger.debug(f'\nevents with clashes added:\n{pformat(events)}')
    logger.info(f'Added clashes for {len(clashes)} event(s).')


def add_unsuitability_to_events(events, slots, unsuitability):
    for event, unsuitable_slots in unsuitability.items():
        events[event].add_unavailability(
            *[slots[s] for s in unsuitable_slots if unsuitable_slots])
    # logger.debug(f'\nevents with unsuitability added:\n{pformat(events)}')
    logger.info(
        f'Added unavailability for {len(unsuitability)} event(s) due to '
        'venue suitability.')


def add_allocations(events, slots, solution, allocations):
    for allocation in allocations:
        events.append(allocation['event'])

    for allocation in allocations:
        event_idx = events.index(allocation['event'])
        slot_idx = slots.index(allocation['slot'])
        solution.append((event_idx, slot_idx))

    logger.info(f'Added {len(allocations)} pre-allocated event(s) to solution')



"""Procedures to define the Command Line Interface (cli)"""
from pathlib import Path

import click
from conference_scheduler.scheduler import event_schedule_difference
from conference_scheduler.converter import solution_to_schedule
from conference_scheduler.validator import (
    is_valid_solution, solution_violations)
import daiquiri

import scheduler.calculate as calc
from scheduler.decorators import timed
import scheduler.define as defn
from scheduler import convert, io, logging, session

logger = daiquiri.getLogger(__name__)


@click.version_option(message='%(prog)s %(version)s :: UK Python Association')
@click.group()
@click.option(
    '--verbosity', '-v', default='info',
    type=click.Choice(['critical', 'error', 'warning', 'info', 'debug']),
    help='Logging verbosity')
def scheduler(verbosity):
    logging.setup(verbosity)


@scheduler.command()
@click.option(
    '--algorithm', '-a', default='pulp_cbc_cmd',
    type=click.Choice(
        ['pulp_cbc_cmd', 'glpk', 'hill_climber', 'simulated_annealing']),
    help='Solver algorithm')
@click.option(
    '--objective', '-o', default=None,
    type=click.Choice(['efficiency', 'equity', 'consistency']),
    help='Objective Function')
@click.option('--diff/--no-diff', default=False, help='Show schedule diff')
@click.option(
    '--input_dir', '-i', default=None, help='Directory for input files')
@click.option(
    '--solution_dir', '-s', default=None, help='Directory for solution files')
@click.option(
    '--build_dir', '-b', default=None, help='Directory for output yaml files')
@timed
def build(algorithm, objective, diff, input_dir, solution_dir, build_dir):
    if input_dir:
        session.folders['input'] = Path(input_dir)

    if solution_dir:
        session.folders['solution'] = Path(solution_dir)

    if build_dir:
        session.folders['build'] = Path(build_dir)

    resources = defn.resources()
    slots = defn.slots(resources)
    events = defn.events(resources)
    unavailability = defn.unavailability(resources, slots)
    clashes = defn.clashes(resources)
    unsuitability = defn.unsuitability(resources, slots)
    allocations = defn.allocations(resources)

    defn.add_unavailability_to_events(events, slots, unavailability)
    defn.add_clashes_to_events(events, clashes)
    defn.add_unsuitability_to_events(events, slots, unsuitability)

    kwargs = {}
    if objective == 'consistency' or diff:
        original_solution = io.import_solution()
        revised_solution = [
            item for item in original_solution
            if item[0] < len(events)]
        original_schedule = solution_to_schedule(
            revised_solution, events, slots)

    if objective == 'consistency':
        diff = True
        kwargs['original_schedule'] = original_schedule

    solution = calc.solution(events, slots, algorithm, objective, **kwargs)

    if diff:
        schedule = solution_to_schedule(solution, events, slots)
        event_diff = event_schedule_difference(schedule, original_schedule)
        logger.debug(f'\nevent_diff:')
        for item in event_diff:
            logger.debug(f'{item.event.name} has moved from {item.old_slot.venue} at {item.old_slot.starts_at} to {item.new_slot.venue} at {item.new_slot.starts_at}')

    if solution is not None:
        defn.add_allocations(events, slots, solution, allocations)
        logger.debug(convert.schedule_to_text(solution, events, slots))
        io.export_solution_and_definition(resources, events, slots, solution)
        io.build_output(resources, events, slots, solution)


@scheduler.command()
@click.option(
    '--solution_dir', '-s', default=None, help='Directory for solution files')
@timed
def validate(solution_dir):
    if solution_dir:
        session.folders['solution'] = Path(solution_dir)

    solution = io.import_solution()
    definition = io.import_schedule_definition()
    logger.info('Validating schedule...')
    if is_valid_solution(solution, definition['events'], definition['slots']):
        logger.info('Imported solution is valid')
    else:
        for v in solution_violations(
                solution, definition['events'], definition['slots']):
            logger.error(v)


@scheduler.command()
@click.option(
    '--solution_dir', '-s', default=None, help='Directory for solution files')
@click.option(
    '--build_dir', '-b', default=None, help='Directory for output yaml files')
@timed
def rebuild(solution_dir, build_dir):
    if solution_dir:
        session.folders['solution'] = Path(solution_dir)

    if build_dir:
        session.folders['build'] = Path(build_dir)

    solution = io.import_solution()
    definition = io.import_schedule_definition()
    logger.info('Validating schedule...')
    if is_valid_solution(solution, definition['events'], definition['slots']):
        logger.info('Imported solution is valid')
        io.build_output(
            definition['resources'], definition['events'],
            definition['slots'], solution)
    else:
        for v in solution_violations(
                solution, definition['events'], definition['slots']):
            logger.error(v)



import functools
import time
import daiquiri

logger = daiquiri.getLogger(__name__)


def timed(func):
    @functools.wraps(func)
    def wrapped(*args, **kwargs):
        start_time = time.time()
        func_return = func(*args, **kwargs)
        elapsed_time = time.time() - start_time
        logger.info(f'Completed in {round(elapsed_time, 2)}s')
        return func_return
    return wrapped



"""Utility functions to convert a solution to human readable form"""
from conference_scheduler import converter


def schedule_to_text(solution, events, slots):
    schedule = converter.solution_to_schedule(solution, events, slots)
    schedule.sort(key=lambda item: item.slot.starts_at)
    message = 'Schedule:\n\n'
    for item in schedule:
        message += (
            f'{item.event.name} at {item.slot.starts_at} in '
            f'{item.slot.venue}\n')
    return message



import daiquiri
from conference_scheduler import scheduler
from conference_scheduler.heuristics import hill_climber
from conference_scheduler.heuristics import simulated_annealing
from conference_scheduler.lp_problem.objective_functions import (
    efficiency_capacity_demand_difference,
    equity_capacity_demand_difference,
    number_of_changes)
from pulp import GLPK
from pulp import PULP_CBC_CMD

logger = daiquiri.getLogger(__name__)

solvers = {
    'pulp_cbc_cmd': {
        'function': scheduler.solution,
        'kwargs': {
            'solver': PULP_CBC_CMD(msg=False)}},
    'glpk': {
        'function': scheduler.solution,
        'kwargs': {
            'solver': GLPK(msg=False)}},
    'hill_climber': {
        'function': scheduler.heuristic,
        'kwargs': {
            'algorithm': hill_climber}},
    'simulated_annealing': {
        'function': scheduler.heuristic,
        'kwargs': {
            'algorithm': simulated_annealing}}}

objectives = {
    'efficiency': efficiency_capacity_demand_difference,
    'equity': equity_capacity_demand_difference,
    'consistency': number_of_changes}


def solution(events, slots, solver, objective=None, **kwargs):
    message = f'Scheduling conference using {solver} solver'
    if objective is not None:
        message += f' and optimising {objective}'
    logger.info(f'{message}...')

    objective_function = objectives.get(objective, None)
    full_kwargs = {
        'events': events,
        'slots': slots,
        'objective_function': objective_function,
        **solvers[solver]['kwargs'],
        **kwargs}

    try:
        solution = solvers[solver]['function'](**full_kwargs)
    except ValueError:
        logger.error('No valid solution found')
        solution = None

    return solution



import csv
import pickle
from datetime import datetime
from pathlib import Path
from pprint import pformat
import shutil

import daiquiri
from conference_scheduler import converter
from ruamel.yaml import YAML
from slugify import slugify

from scheduler import session

logger = daiquiri.getLogger(__name__)
yaml = YAML(typ='safe')
yaml.default_flow_style = False


def import_yaml():
    """Import all yaml files in the given folder into a single resources
    dict"""
    yaml_resources = {}
    yaml_files = [
        path for path in session.folders['input'].iterdir()
        if path.suffix == '.yml']
    for path in yaml_files:
        with path.open('r') as file:
            yaml_resources[path.stem] = yaml.load(file)
    logger.debug(f'\nreources:\n{pformat(yaml_resources)}')
    return yaml_resources


def import_proposals(resources):
    """Import the proposals data from a .csv file"""
    proposals = []
    with Path(session.folders['input'], 'proposals.csv').open('r') as file:
        reader = csv.DictReader(file)
        for row in reader:
            if row['session_type'] in resources['event_types']:
                event_type = row['session_type']
                proposals.append({
                    'title': row['title'],
                    'duration': int(row['duration']),
                    'demand': float(row.get('demand', 0)),
                    'person': slugify(row['name']),
                    'tags': [row['tag']] if row['tag'] != '' else [],
                    'event_type': event_type})
    logger.debug(f'\nreources:\n{pformat(proposals)}')
    return proposals


def import_solution():
    """Import a previously computed schedule from a .csv file"""
    csv_file = Path(session.folders['solution'], 'schedule.csv')
    logger.info(f'Importing schedule from {csv_file}')
    solution = []
    with Path(csv_file).open('r') as file:
        reader = csv.DictReader(file)
        for row in reader:
            solution.append((
                int(row['event_index']),
                int(row['slot_index'])))
        logger.debug(f'\nreources:\n{pformat(solution)}')
    return solution


def import_schedule_definition():
    """Import previously pickled schedule"""
    pickle_file = Path(session.folders['solution'], 'scheduler.pickle')
    logger.info(
        f'Importing resources, events, slots and schedule from {pickle_file}')
    with pickle_file.open('rb') as f:
        bundle = pickle.load(f)
    return bundle


def pickle_solution_and_definition(resources, events, slots, solution):
    """Store the computed solution, the resources dict and the associated
    events and slots lists in pickle format"""
    pickle_file = Path(session.folders['solution'], 'scheduler.pickle')
    logger.info(
        f'Pickling resources, events, slots and schedule to {pickle_file}')
    bundle = {
        'resources': resources,
        'events': events,
        'slots': slots,
        'solution': solution
    }
    with pickle_file.open('wb') as f:
        pickle.dump(bundle, f, pickle.HIGHEST_PROTOCOL)


def export_schedule(solution, events, slots):
    """Write a human readable .csv file of the computed solution"""
    csv_file = Path(session.folders['solution'], 'schedule.csv')
    logger.info(f'Exporting schedule to {csv_file}')

    schedule = converter.solution_to_schedule(solution, events, slots)
    scheduled_items = [
        {
            'event_index': events.index(item.event),
            'event': f'{item.event.name}',
            'slot_index': slots.index(item.slot),
            'slot': f'{item.slot.starts_at} {item.slot.venue}'
        }
        for item in schedule
    ]
    with csv_file.open('w') as f:
        writer = csv.DictWriter(
            f, fieldnames=['event_index', 'event', 'slot_index', 'slot'])
        writer.writeheader()
        for item in scheduled_items:
            writer.writerow(item)


def export_solution_and_definition(resources, events, slots, solution):
    session.folders['solution'].mkdir(exist_ok=True)
    pickle_solution_and_definition(resources, events, slots, solution)
    export_schedule(solution, events, slots)


def build_output(resources, events, slots, solution):
    """Create the yaml files required by the conference django-amber based
    website for display of the programme"""
    logger.info(f'Creating output files in {session.folders["build"]}...')
    shutil.rmtree(session.folders['build'], ignore_errors=True)
    session.folders['build'].mkdir()

    day_format = '%A %-d'
    start_format = '%H:%M'

    for item in solution:
        slot_time = slots[item[1]].starts_at
        day = slot_time.strftime(day_format)
        start_time = slot_time.strftime(start_format)
        venue = slots[item[1]].venue

        content = {
            'chair': None,
            'room': venue,
            'date': day,
            'time': start_time,
            'title': events[item[0]].name,
        }

        folder = Path(session.folders['build'], day, venue)
        folder.mkdir(parents=True, exist_ok=True)
        with Path(folder, start_time).open('a') as f:
            yaml.dump(content, f)



import sys
from pathlib import Path

import daiquiri

from scheduler import session


def setup(verbosity):
    log_file = Path(session.folders['solution'], 'scheduler.log')
    try:
        log_file.unlink()
    except FileNotFoundError:
        pass

    daiquiri.setup(
        level='DEBUG',
        outputs=(
            daiquiri.output.Stream(sys.stdout, level=verbosity.upper()),
            daiquiri.output.File(log_file, level='DEBUG'))
    )



import sys
from pathlib import Path

this = sys.modules[__name__]

this.folders = {
    'input': Path(Path.cwd(), 'input'),
    'solution': Path(Path.cwd(), 'solution'),
    'build': Path(Path.cwd(), 'build')
}



import io
import re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup


def read(*names, **kwargs):
    return io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ).read()


setup(
    name='conference-scheduler-cli',
    version='0.8.2',
    license='MIT license',
    description='A command line tool to manage the schedule for a conference',
    long_description='%s\n%s' % (
        re.compile('^.. start-badges.*^.. end-badges', re.M | re.S).sub(
            '', read('README.rst')),
        re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.rst'))
    ),
    author='The UK Python Association',
    author_email=('owen.campbell@uk.python.org'),
    url='https://github.com/pyconuk/conferencescheduler-cli',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    entry_points={
        'console_scripts': [
            'scheduler = scheduler.cli:scheduler',
        ],
    },
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Utilities',
    ],
    install_requires=[
        'click >= 6.7',
        'conference-scheduler >= 2.1.2',
        'daiquiri >= 1.2.1',
        'python-slugify >= 1.2.4',
        'ruamel.yaml >= 0.15.23'
    ],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    python_requires=">=3.6",
)
